#!/usr/bin/python3
"""Easy HTTP/HTTPS server."""

import logging
import http.server
import os
import socketserver
import ssl
import threading


class UpgradeServer(object):
    HOST = 'localhost'
    HTTP_PORT = 8000
    HTTPS_PORT = 4443

    def serve(self):
        # Get path to certificate and key files
        cert_path = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            os.path.pardir,
            'data',
            'cert.pem'))
        logging.info('cert file: {}'.format(cert_path))
        key_path = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            os.path.pardir,
            'data',
            'key.pem'))
        logging.info('key file: {}'.format(key_path))

        # Change working directory to serve only desired files
        server_directory = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            os.path.pardir,
            'server-files'))
        logging.info('Server directory: {}'.format(server_directory))
        os.chdir(server_directory)

        def serve_http():
            httpd = socketserver.TCPServer(
                (self.HOST, self.HTTP_PORT),
                http.server.SimpleHTTPRequestHandler)
            logging.info('Running HTTP server in {}:{}'
                         .format(self.HOST, self.HTTP_PORT))
            httpd.serve_forever()

        def serve_https(cert_path, key_path):
            # Thanks pitti!
            # http://www.piware.de/2011/01/creating-an-https-server-in-python/
            httpd = http.server.HTTPServer(
                (self.HOST, self.HTTPS_PORT),
                http.server.SimpleHTTPRequestHandler)
            httpd.socket = ssl.wrap_socket(
                httpd.socket,
                certfile=cert_path,
                keyfile=key_path,
                server_side=True)
            logging.info('Running HTTPS server in {}:{}'
                         .format(self.HOST, self.HTTPS_PORT))
            httpd.serve_forever()

        threading.Thread(target=serve_http).start()
        threading.Thread(target=serve_https,
                         args=(cert_path, key_path)).start()


def main():
    server = UpgradeServer()
    server.serve()


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.INFO,
        format='%(asctime)s %(levelname)s: %(message)s')
    main()
